*--------------------------------------------------------------------------------------------------
* File: 	C:\Projects\Stata Weighting Book\program files\3 Nonresponse\boostNR.do
* Project:  Stata weighting book
* Date: 	07/28/2015
* Revised:  04/02/2017
*           Incorporated M. Schonlau advice to run boost for several choices of interaction and shrink. 
*	    	Then pick best one.
* Author:	R. Valliant
* Purpose:	Illustrate boosting for NR adjustment
*--------------------------------------------------------------------------------------------------

log using "C:\Projects\Stata Weighting Book\Program files\3 Nonresponse\Ex.3.6_boostNR.smcl", replace

version 14
set more off

use "C:\Projects\Stata Weighting Book\Data\nhis\nhis.dta", clear

    * randomize order of the file
set seed 526544083
generate double u1 = runiform()
sort u1

*--------------------------------------------------------------------------------------------------
	* generate dummies by hand to use in boost
tab hisp
gen hisp1 = (hisp == 1)
tab hisp1
tab race
gen race1 = (race == 1)
gen race2 = (race == 2)
gen race3 = (race == 3)

tab race1 
tab race2 
tab race3

tab parents_r
gen parents_r1 = (parents_r == 1)
tab parents_r1

tab educ_r
gen educ_r1 = (educ_r == 1)
gen educ_r2 = (educ_r == 2)
gen educ_r3 = (educ_r == 3)
gen educ_r4 = (educ_r == 4)
tab educ_r1
tab educ_r2
tab educ_r3
tab educ_r4

*--------------------------------------------------------------------------------------------------
	* logistic predictions

logit resp age i.hisp i.race i.parents_r i.educ_r      
predict predp, pr
label variable predp "Logistic predictions"	
    * check logistic classification accuracies
generate double u2 = runiform()
gen logcl = "NR"
replace logcl = "R" if u2 <= predp
*gen logcl = round(predp)
tab logcl resp, column V

	* Try "majority vote" assignment to R or NR
gen log_pred = round(predp)
gen log_accuracy = log_pred == resp
tab log_accuracy 
tab log_pred resp, column V

gen lwt = 1/predp
egen m_lwt = mean(lwt)
egen sd_lwt = sd(lwt)
display 1 + (sd_lwt / m_lwt)^2
*	1.0108732

    * save only logistic predicted probs
gen logistpreds = predp
label variable logistpreds "Logistic predictions"	
*keep ID logistpreds     
*save "C:\Projects\Stata Weighting Book\Data\nhis\logistpreds.dta", replace

*--------------------------------------------------------------------------------------------------
	* boost using dummy vars
	* Per Matt Schonlau: do a grid search over these combos:
	* 	interaction 3,5,7	(default = 5)
	*   shrink 0.1 0.01 0.001	(default = 0.01)
	*
	* Best choice is (7) interaction 7; shrink 0.1
	* 	Go directly to this choice to reproduce result in book
*--------------------------------------------------------------------------------------------------

		* (1) interaction 3; shrink 0.1
boost resp age hisp1 race1 race2 race3 parents_r1 educ_r1 educ_r2 educ_r3 educ_r4, distribution(logistic) trainfraction(0.8) interaction(3) shrink(0.1) predict(boostpI3_S0p1) bag(0.7) seed(1183238) influence
label variable boostpI3_S0p1 "Boost predictions I=3 S=0.1"
    * check boost classification accuracies
gen double u3 = runiform()
gen boostcl = "NR"
replace boostcl = "R" if u3 <= boostpI3_S0p1
tab boostcl resp, column V

		* (2) interaction 3; shrink 0.01
boost resp age hisp1 race1 race2 race3 parents_r1 educ_r1 educ_r2 educ_r3 educ_r4, distribution(logistic) trainfraction(0.8) interaction(3) shrink(0.01) predict(boostpI3_S0p01) bag(0.7) seed(884874224) influence
label variable boostpI3_S0p01 "Boost predictions I=3 S=0.01"
    * check boost classification accuracies
replace u3 = runiform()
replace boostcl = "NR"
replace boostcl = "R" if u3 <= boostpI3_S0p01
tab boostcl resp, column V

		* (3) interaction 3; shrink 0.001
boost resp age hisp1 race1 race2 race3 parents_r1 educ_r1 educ_r2 educ_r3 educ_r4, distribution(logistic) trainfraction(0.8) interaction(3) shrink(0.001) predict(boostpI3_S0p001) bag(0.7) seed(1180709481) influence
label variable boostpI3_S0p001 "Boost predictions I=3 S=0.001"
    * check boost classification accuracies
replace u3 = runiform()
replace boostcl = "NR"
replace boostcl = "R" if u3 <= boostpI3_S0p001
tab boostcl resp, column V

		* (4) interaction 5; shrink 0.1
boost resp age hisp1 race1 race2 race3 parents_r1 educ_r1 educ_r2 educ_r3 educ_r4, distribution(logistic) trainfraction(0.8) interaction(5) shrink(0.1) predict(boostpI5_S0p1) bag(0.7) seed(864287359) influence
label variable boostpI5_S0p1 "Boost predictions I=5 S=0.1"
    * check boost classification accuracies
replace u3 = runiform()
replace boostcl = "NR"
replace boostcl = "R" if u3 <= boostpI5_S0p1
tab boostcl resp, column V

		* (5) interaction 5; shrink 0.01
boost resp age hisp1 race1 race2 race3 parents_r1 educ_r1 educ_r2 educ_r3 educ_r4, distribution(logistic) trainfraction(0.8) interaction(5) shrink(0.01) predict(boostpI5_S0p01) bag(0.7) seed(1183238) influence
label variable boostpI5_S0p01 "Boost predictions I=5 S=0.01"
    * check boost classification accuracies
replace u3 = runiform()
replace boostcl = "NR"
replace boostcl = "R" if u3 <= boostpI5_S0p01
tab boostcl resp, column V

		* (6) interaction 5; shrink 0.001
boost resp age hisp1 race1 race2 race3 parents_r1 educ_r1 educ_r2 educ_r3 educ_r4, distribution(logistic) trainfraction(0.8) interaction(5) shrink(0.001) predict(boostpI5_S0p001) bag(0.7) seed(507050408) influence
label variable boostpI5_S0p001 "Boost predictions I=5 S=0.001"
   * check boost classification accuracies
replace u3 = runiform()
replace boostcl = "NR"
replace boostcl = "R" if u3 <= boostpI5_S0p001
tab boostcl resp, column V

*--------------------------------------------------------------------------------------------------
		* (7) interaction 7; shrink 0.1
        * this is best choice
boost resp age hisp1 race1 race2 race3 parents_r1 educ_r1 educ_r2 educ_r3 educ_r4, distribution(logistic) trainfraction(0.8) interaction(7) shrink(0.1) predict(boostpI7_S0p1) bag(0.7) seed(528179368) influence
label variable boostpI7_S0p1 "Boost predictions I=7 S=0.1"

	* Try "majority vote" assignment to R or NR
gen boost_pred = round(boostpI7_S0p1)
gen accuracy = boost_pred == resp
tab accuracy
tab boost_pred resp, column V
tab boost_pred resp if _n <= 3128, column V
tab boost_pred resp if _n > 3128, column V

gen bwt = 1/boostp
egen m_bwt = mean(bwt)
egen sd_bwt = sd(bwt)
display 1 + (sd_bwt / m_bwt)^2
*	1.0077517 a little smaller than logistic but not much

save "C:\Projects\Stata Weighting Book\Data\nhis\boostpreds.dta", replace

*--------------------------------------------------------------------------------------------------
		* (8) interaction 7; shrink 0.01
boost resp age hisp1 race1 race2 race3 parents_r1 educ_r1 educ_r2 educ_r3 educ_r4, distribution(logistic) trainfraction(0.8) interaction(7) shrink(0.01) predict(boostpI7_S0p01) bag(0.7) seed(2054385476) influence
label variable boostpI7_S0p01 "Boost predictions I=7 S=0.01"
   * check boost classification accuracies
replace u3 = runiform()
replace boostcl = "NR"
replace boostcl = "R" if u3 <= boostpI7_S0p01
tab boostcl resp, column V

		* (9) interaction 7; shrink 0.001
boost resp age hisp1 race1 race2 race3 parents_r1 educ_r1 educ_r2 educ_r3 educ_r4, distribution(logistic) trainfraction(0.8) interaction(7) shrink(0.001) predict(boostpI7_S0p001) bag(0.7) seed(760114657) influence
label variable boostpI7_S0p001 "Boost predictions I=3 S=0.001"
   * check boost classification accuracies
replace u3 = runiform()
replace boostcl = "NR"
replace boostcl = "R" if u3 <= boostpI7_S0p001
tab boostcl resp, column V

log close
