*--------------------------------------------------------------------------------------------------
* File: 	C:\Projects\Stata Weighting Book\Program files\7 Special Cases\Ex.7.2_svywt.test.linear.do
* Project:      Stata Weighting book
* Date: 	03/08/2017
* Author:	R. Valliant
* Purpose:	Illustrate test for whether OLS and survey weighted regression parm estimates are equal
*			for linear regression
*--------------------------------------------------------------------------------------------------

log using "C:\Projects\Stata Weighting Book\Program files\7 Special Cases\Ex.7.2_svywt.test.linear.smcl", replace

set more off
use "C:\Projects\Stata Weighting Book\Data\RECS\recs2009.dta", clear

ssc install dlist

gen one = 1
gen totbtucolM = TOTALBTUCOL/1000

gen w_one = NWEIGHT*one
gen w_cdd65 = NWEIGHT*CDD65

gen w_aiazone1 = NWEIGHT*(AIA_Zone == 1)
gen w_aiazone2 = NWEIGHT*(AIA_Zone == 2)
gen w_aiazone3 = NWEIGHT*(AIA_Zone == 3)
gen w_aiazone4 = NWEIGHT*(AIA_Zone == 4)
gen w_aiazone5 = NWEIGHT*(AIA_Zone == 5)

gen w_totrooms = NWEIGHT*TOTROOMS 

recode COOLTYPE (-2 = 0), gen(rCOOLTYPE)
gen w_cooltype1 = NWEIGHT*(COOLTYPE==1)
gen w_cooltype2 = NWEIGHT*(COOLTYPE==2)
gen w_cooltype3 = NWEIGHT*(COOLTYPE==3)
gen w_cooltype0 = NWEIGHT*(COOLTYPE < 0)

gen w_totsqft = NWEIGHT*TOTSQFT

	* run wtd and unwtd models separately
	* wtd regn
svyset [pw=NWEIGHT], vce(brr) brrw(brr_weight_1 - brr_weight_244) fay(0.5) dof(244) mse
svy: regress totbtucolM CDD65 i.AIA_Zone TOTROOMS i.rCOOLTYPE TOTSQFT

	* compute R2 based on code from a post by Steve Samuels
	* http://www.stata.com/statalist/archive/2008-10/msg00850.html
predict double resid, residual
svy: mean totbtucolM resid 

gen v1 = e(N)*el(e(V_srs),1,1)
gen v2 = e(N)*el(e(V_srs),2,2)
gen adjr2= (1-v2/v1)
label var adjr2 "Adjusted R-Sq"
format adjr2 %8.4f
dlist adjr2 in 1, name(0)
	
	* unwtd regn
regress totbtucolM CDD65 i.AIA_Zone TOTROOMS i.rCOOLTYPE TOTSQFT

	* extended regression
#delimit ;
 regress TOTALBTUCOL CDD65 i.AIA_Zone TOTROOMS i.rCOOLTYPE TOTSQFT
		w_one w_cdd65
		w_aiazone1 w_aiazone2 w_aiazone3 w_aiazone4 w_aiazone5
		w_totrooms
		w_cooltype1 w_cooltype2 w_cooltype3 w_cooltype0 ;
#delimit cr

note: w_aiazone5 omitted because of collinearity
note: w_cooltype0 omitted because of collinearity

#delimit ;
test w_one w_cdd65 
		w_aiazone1 w_aiazone2 w_aiazone3 w_aiazone4 w_aiazone5 
		w_totrooms 
		w_cooltype1 w_cooltype2 w_cooltype3 w_cooltype0 ;
#delimit cr

log close
